#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.prod_svc_detect_in_vulnvt import (
    CheckProdSvcDetectInVulnvt,
)

from . import PluginTestCase


class CheckProdSVCDetectInVulnvtTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"0.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckProdSvcDetectInVulnvt(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckProdSvcDetectInVulnvt(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_nok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'script_family("Product detection");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckProdSvcDetectInVulnvt(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT has a severity but is placed in the family 'Product detection' "
            "which is not allowed for this VT. Please split this VT into a "
            "separate Product/Service detection and Vulnerability-VT.",
            results[0].message,
        )

    def test_nok2(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'script_family("Product detection");\n'
            "register_product(cpe:cpe);\n"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckProdSvcDetectInVulnvt(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 2)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT has a severity but is placed in the family 'Product detection' "
            "which is not allowed for this VT. Please split this VT into a "
            "separate Product/Service detection and Vulnerability-VT.",
            results[0].message,
        )
        self.assertEqual(
            "VT has a severity but is using the function 'register_product' "
            "which is not allowed for this VT. Please split this VT into a "
            "separate Product/Service detection and Vulnerability-VT.",
            results[1].message,
        )



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.solution_text import CheckSolutionText

from . import PluginTestCase

CORRECT_WILLNOTFIX_SUGGESTION = (
    "The VT with solution type 'WillNotFix' is using an incorrect syntax in "
    "the solution text. Please use one of these (EXACTLY):\n  "
    'script_tag(name:"solution", value:"No known solution was made available '
    "for at least one year\n  since the disclosure of this vulnerability. "
    "Likely none will be provided anymore. General solution\n  options are to "
    "upgrade to a newer release, disable respective features, remove the "
    'product or\n  replace the product by another one.");\n\n  '
    'script_tag(name:"solution", value:"No solution was made available by the '
    "vendor. General solution\n  options are to upgrade to a newer release, "
    "disable respective features, remove the product or\n  replace the product "
    'by another one.");\n\n  script_tag(name:"solution", value:"No solution '
    "was made available by the vendor.\n\n  Note: <add a specific note for the "
    'reason here>.");\n\n  script_tag(name:"solution", value:"No solution was '
    "made available by the vendor.\n\n  Vendor statement: <add specific vendor "
    'statement here>.");\n\n  script_tag(name:"solution", value:"No solution '
    "is required.\n\n  Note: <add a specific note for the reason here, e.g. "
    'CVE was disputed>.");'
)


class CheckSolutionTextTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"solution_type", value:"NoneAvailable");\n'
            'script_tag(name:"solution", value:"No known solution is '
            "available as of 01st September, 2021.\n  Information "
            "regarding this issue will be updated once solution details "
            'are available.");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok2(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"solution_type", value:"WillNotFix");\n'
            'script_tag(name:"solution", '
            'value:"No solution was made available by the vendor.\n\n  Note: '
            '<add a specific note for the reason here>.");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok3(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"solution_type", value:"WillNotFix");\n'
            'script_tag(name:"solution", '
            'value:"No solution was made available by the vendor.\n\n  Vendor '
            'statement: <add specific vendor statement here>.");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_nok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = 'script_tag(name:"solution_type", value:"NoneAvailable");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The VT with solution type 'NoneAvailable' is using an "
            "incorrect syntax in the solution text. Please use (EXACTLY):\n"
            '  script_tag(name:"solution", value:"No known solution is '
            "available as of dd(st|nd|rd|th) mmmmmmmm, yyyy.\n  Information "
            "regarding this issue will be updated once solution details "
            'are available.");',
            results[0].message,
        )

    def test_nok2(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = 'script_tag(name:"solution_type", value:"WillNotFix");\n'
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            CORRECT_WILLNOTFIX_SUGGESTION,
            results[0].message,
        )

    def test_nok3(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"solution_type", value:"WillNotFix");\n'
            'script_tag(name:"solution", '
            'value:"No solution was made available by the vendor.\n\n  Notice: '
            '<add a specific note for the reason here>.");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            CORRECT_WILLNOTFIX_SUGGESTION,
            results[0].message,
        )

    def test_nok4(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"solution_type", value:"WillNotFix");\n'
            'script_tag(name:"solution", '
            'value:"No solution was made available by the vendor.\n\n  Vendor '
            'statment: <add specific vendor statement here>.");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckSolutionText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            CORRECT_WILLNOTFIX_SUGGESTION,
            results[0].message,
        )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckScriptCopyright(FileContentPlugin):
    name = "check_script_copyright"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This plugin checks a VT for the correct use of the copyright text.

        Prior to this plugin, most VTs had used a
        'script_copyright("This script is Copyright (C) [...]");'
        tag, however the introductory text ("This script is") is to be discarded
        from now on.

        In addition this plugin will also report if the syntax of the
        'script_copyright();' tag is generally missing or malformed.

        Args:
            nasl_file: The VT that shall be checked
            file_content: str representing the file content
        """
        if nasl_file.suffix == ".inc":
            return

        if not re.search(
            r'script_copyright\("Copyright \(C\) [0-9]{4}', file_content
        ):
            yield LinterError(
                "The VT is using an incorrect syntax for its "
                "copyright statement. Please start (EXACTLY) with: "
                "'script_copyright(\"Copyright (C)' followed by the year "
                "(matching the one in creation_date) and the author/company.",
                file=nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import re
from argparse import ArgumentParser, Namespace
from pathlib import Path
from subprocess import CalledProcessError
from typing import Iterable, List

from pontos.terminal.terminal import ConsoleTerminal

from troubadix.argparser import file_type
from troubadix.standalone_plugins.changed_packages.marker import (
    AddedEpoch,
    AddedRelease,
    AddedUdeb,
    ChangedUpdate,
    DroppedArchitecture,
)
from troubadix.standalone_plugins.changed_packages.package import (
    Package,
    Reasons,
)
from troubadix.standalone_plugins.common import get_merge_base, git

PACKAGE_CHECK_PATTERN = re.compile(
    r'isdpkgvuln\(pkg:"(?P<package>[^"]+)", ver:"(?P<version>[^"]+)", '
    r'rls:"(?P<release>[^"]+)"\)'
)


def compare(old_content: str, content: str):
    old_packages = get_packages(old_content)
    packages = get_packages(content)

    missing_packages = sorted(old_packages.difference(packages))
    new_packages = sorted(packages.difference(old_packages))

    AddedEpoch.mark(missing_packages, new_packages)
    AddedRelease.mark(old_packages, new_packages)
    AddedUdeb.mark(new_packages)
    ChangedUpdate.mark(missing_packages, new_packages)
    DroppedArchitecture.mark(missing_packages, new_packages)

    return missing_packages, new_packages


def filter_reasons(packages: List[Package], reasons: Iterable[Reasons]):
    return [
        package
        for package in packages
        if not package.reasons
        or any([reason not in reasons for reason in package.reasons])
    ]


def print_results(
    missing_packages: List[Package],
    new_packages: List[Package],
    file: Path,
    terminal: ConsoleTerminal,
):
    terminal.warning(f"Packages for {file} differ")

    if missing_packages:
        terminal.print("Missing packages")
        print_packages(missing_packages, terminal)

    if new_packages:
        terminal.print("New packages")
        print_packages(new_packages, terminal)


def print_packages(
    packages: List[Package],
    terminal: ConsoleTerminal,
):
    with terminal.indent():
        for package in packages:
            terminal.print(f"{package}")


def get_packages(content: str):
    package_checks = PACKAGE_CHECK_PATTERN.findall(content)
    result = {
        Package(name=name, version=version, release=release)
        for name, version, release in package_checks
    }

    if len(result) != len(package_checks):
        raise ValueError("There are duplicate checks. Cannot compare.")

    return result


def parse_args() -> Namespace:
    parser = ArgumentParser(
        description="Check for changed packages in dpkg-based LSCs",
    )
    parser.add_argument(
        "--files",
        nargs="+",
        type=file_type,
        default=[],
        required=True,
        help="List of files to check.",
    )
    parser.add_argument(
        "--start-commit",
        type=str,
        required=False,
        help=(
            "The commit before the changes to check have been introduced. "
            "If the files have been renamed before, choose that commit. "
            "Defaults to the merge-base with main"
        ),
        default=get_merge_base("main", "HEAD"),
    )
    parser.add_argument(
        "--hide-equal",
        action="store_true",
        help="Omit log message, if a file has equal checks",
    )
    parser.add_argument(
        "--hide-reasons",
        nargs="*",
        action="extend",
        default=[],
        type=Reasons.from_cli_argument,
        choices=list(Reasons),
        help="Disable the output for packages that changed for a given reason",
    )

    return parser.parse_args()


def main():
    args = parse_args()
    hide_reasons = set(args.hide_reasons)
    terminal = ConsoleTerminal()

    terminal.info(
        f"Checking {len(args.files)} file(s) from {args.start_commit} to HEAD"
    )

    for file in args.files:
        try:
            old_content = git("show", f"{args.start_commit}:{file}")
            content = git("show", f"HEAD:{file}")
            missing_packages, new_packages = compare(old_content, content)
        except CalledProcessError:
            terminal.error(
                f"Could not find {file} at {args.start_commit} or HEAD"
            )
            continue
        except ValueError as e:
            terminal.error(f"Error while handling {file}: {e}")
            continue

        if not missing_packages and not new_packages:
            if not args.hide_equal:
                terminal.info(f"{file} has equal checks")
            continue

        missing_packages = filter_reasons(missing_packages, hide_reasons)
        new_packages = filter_reasons(new_packages, hide_reasons)

        if not missing_packages and not new_packages and hide_reasons:
            terminal.info(f"Packages for {file} differ, but reasons are hidden")
            continue

        print_results(
            missing_packages,
            new_packages,
            file,
            terminal,
        )


if __name__ == "__main__":
    main()



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from typing import Iterator

from validators import url

from troubadix.helper.patterns import get_xref_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

ALLOWED_URLS = [
    "https://lists.apache.org/thread.html/e1ef853fc0079cdb55be"
    "fbd2dac042934e49288b476d5f6a649e5da2@<announce.tomcat.apache.org>",
    "https://lists.apache.org/thread.html/e1ef853fc0079cdb55be"
    "fbd2dac042934e49288b476d5f6a649e5da2@<announce.tomcat.apache.org>",
    "https://m0ze.ru/vulnerability/[2021-05-26]-[WordPress]-[C"
    "WE-79]-WP-Reset-WordPress-Plugin-v1.86.txt",
    "http://yehg.net/lab/pr0js/advisories/joomla/core/[joomla_"
    "1.0.x~15]_cross_site_scripting",
    "http://yehg.net/lab/pr0js/advisories/eclipse/[eclipse_hel"
    "p_server]_cross_site_scripting",
    "http://core.yehg.net/lab/pr0js/advisories/dll_hijacking/["
    "flash_player]_10.1.x_insecure_dll_hijacking_(dwmapi.dll)",
    "https://lists.apache.org/thread.html/773c93c2d8a6a52bbe9"
    "7610c2b1c2ad205b970e1b8c04fb5b2fccad6@<general.hadoop.apache.org>",
    "http://mail-archives.apache.org/mod_mbox/perl-advocacy/2"
    "00904.mbox/<ad28918e0904011458h273a71d4x408f1ed286c9dfbc@mail.gmail.com>",
    "http://yehg.net/lab/pr0js/advisories/[mybb1.6]_cross_site_scripting",
]


class CheckScriptXrefUrl(FileContentPlugin):
    name = "check_script_xref_url"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Checks if a URL type script_xref call contains a valid URL
        """
        if nasl_file.suffix == ".inc":
            return

        matches = get_xref_pattern(name="URL", value=r".+?").finditer(
            file_content
        )
        for match in matches:
            if match:
                if not match.group("value") in ALLOWED_URLS and not url(
                    match.group("value")
                ):
                    yield LinterError(
                        f"{match.group(0)}: Invalid URL value",
                        file=nasl_file,
                        plugin=self.name,
                    )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from typing import Iterable

from troubadix.standalone_plugins.changed_packages.package import Package


class Marker:
    @staticmethod
    def _find_package(package: Package, packages: Iterable[Package]):
        result = next(
            (
                other_package
                for other_package in packages
                if other_package.name == package.name
                and other_package.version == package.version
                and other_package.release == package.release
            ),
            None,
        )
        return result



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime
from pathlib import Path
from typing import Iterator

from troubadix.helper.patterns import ScriptTag, get_script_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult

LENGTH = 44


class CheckCreationDate(FileContentPlugin):
    name = "creation_date"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        if not "creation_date" in file_content:
            yield LinterError(
                "No creation date has been found.",
                file=nasl_file,
                plugin=self.name,
            )
            return

        tag_pattern = get_script_tag_pattern(ScriptTag.CREATION_DATE)

        # Example: "2017-11-29 13:56:41 +0100 (Wed, 29 Nov 2017)"
        match = tag_pattern.search(file_content)

        if match:
            try:
                date_left = datetime.strptime(
                    match.group("value")[:25], "%Y-%m-%d %H:%M:%S %z"
                )
                # 2017-11-29 13:56:41 +0100 (error if no timezone)
                date_right = datetime.strptime(
                    match.group("value")[27:43], "%a, %d %b %Y"
                )
                week_day_parsed = date_right.strftime("%a")
            except ValueError:
                yield LinterError(
                    "False or incorrectly formatted creation_date.",
                    file=nasl_file,
                    plugin=self.name,
                )
                return

            week_day_str = match.group("value")[27:30]
            # Wed, 29 Nov 2017
            if date_left.date() != date_right.date():
                yield LinterError(
                    "The creation_date consists of two different dates.",
                    file=nasl_file,
                    plugin=self.name,
                )
            # Check correct weekday
            elif week_day_str != week_day_parsed:
                formatted_date = week_day_parsed
                yield LinterError(
                    f"Wrong day of week. Please change it from '{week_day_str}"
                    f"' to '{formatted_date}'.",
                    file=nasl_file,
                    plugin=self.name,
                )
        else:
            yield LinterError(
                "False or incorrectly formatted creation_date.",
                file=nasl_file,
                plugin=self.name,
            )
            return



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from unittest import TestCase

from troubadix.standalone_plugins.changed_packages.changed_packages import (
    filter_reasons,
    get_packages,
)
from troubadix.standalone_plugins.changed_packages.package import (
    Direction,
    Package,
    Reasons,
)


class ChangedPackagesTestCase(TestCase):
    def test_get_package(self):
        content = """
        ...some NASL...
        if(!isnull(res = isdpkgvuln(pkg:"libwpewebkit-1.0-3", ver:"2.38.3-1~deb11u1", rls:"DEB11"))) {
            report += res;
        }
        if(!isnull(res = isdpkgvuln(pkg:"libwpewebkit-1.0-dev", ver:"2.38.3-1~deb11u1", rls:"DEB11"))) {
            report += res;
        }
        if(!isnull(res = isdpkgvuln(pkg:"libwpewebkit-1.0-doc", ver:"2.38.3-1~deb11u1", rls:"DEB11"))) {
            report += res;
        }
        if(!isnull(res = isdpkgvuln(pkg:"wpewebkit-driver", ver:"2.38.3-1~deb11u1", rls:"DEB11"))) {
            report += res;
        }
        ...some more NASL...
        """

        result = get_packages(content)

        expected_result = {
            Package("libwpewebkit-1.0-3", "2.38.3-1~deb11u1", "DEB11"),
            Package("libwpewebkit-1.0-dev", "2.38.3-1~deb11u1", "DEB11"),
            Package("libwpewebkit-1.0-doc", "2.38.3-1~deb11u1", "DEB11"),
            Package("wpewebkit-driver", "2.38.3-1~deb11u1", "DEB11"),
        }

        self.assertEqual(expected_result, result)

    def test_get_package_duplicate(self):
        content = """
        ...some NASL...
        if(!isnull(res = isdpkgvuln(pkg:"foo", ver:"1.2.3", rls:"DEB10"))) {
            report += res;
        }
        if(!isnull(res = isdpkgvuln(pkg:"bar", ver:"1.2.3", rls:"DEB10"))) {
            report += res;
        }
        if(!isnull(res = isdpkgvuln(pkg:"foo", ver:"1.2.3", rls:"DEB10"))) {
            report += res;
        }
        ...some more NASL...
        """

        with self.assertRaises(Exception):
            get_packages(content)

    def test_filter_reasons(self):
        packages = [
            Package(
                "foo", "1.2.3", "DEB11", {Reasons.ADDED_EPOCH: Direction.ACTIVE}
            ),
            Package(
                "bar",
                "1.2.3",
                "DEB11",
                {
                    Reasons.ADDED_EPOCH: Direction.ACTIVE,
                    Reasons.ADDED_RELEASE: Direction.PASSIVE,
                },
            ),
            Package(
                "baz", "1.2.3", "DEB11", {Reasons.ADDED_UDEB: Direction.ACTIVE}
            ),
        ]

        expected_packages = [
            Package(
                "bar",
                "1.2.3",
                "DEB11",
                {
                    Reasons.ADDED_EPOCH: Direction.ACTIVE,
                    Reasons.ADDED_RELEASE: Direction.PASSIVE,
                },
            ),
            Package(
                "baz", "1.2.3", "DEB11", {Reasons.ADDED_UDEB: Direction.ACTIVE}
            ),
        ]

        result = filter_reasons(packages, [Reasons.ADDED_EPOCH])

        self.assertEqual(expected_packages, result)



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from itertools import chain
from pathlib import Path
from typing import Iterator

from troubadix.helper import ScriptTag, SpecialScriptTag, get_script_tag_pattern
from troubadix.helper.helper import ENTERPRISE_FOLDERS, FEED_VERSIONS
from troubadix.helper.patterns import _get_special_script_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckVTPlacement(FileContentPlugin):
    """The script checks if the passed VT is using one of the
    two following families:

    - script_family("Service detection");
    - script_family("Product detection");

    and is correctly placed into the "root" of the VTs directory.
    """

    name = "check_vt_placement"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """
        Args:
            nasl_file: The VT that shall be checked
            file_content: str representing the file content

        Returns:
            if no problem
        """

        if nasl_file.suffix == ".inc":
            return

        root = self.context.root

        match = _get_special_script_tag_pattern(
            name=SpecialScriptTag.FAMILY.value,
            value=r"(Product|Service) detection",
            flags=re.MULTILINE,
        ).search(file_content)
        if match is None:
            return

        tag_pattern = get_script_tag_pattern(ScriptTag.DEPRECATED)

        match = tag_pattern.search(file_content)
        if match is not None:
            return

        if any(
            (root / vers / nasl_file.name) == nasl_file
            for vers in FEED_VERSIONS
        ):
            return

        for folder in chain(["attic"], ENTERPRISE_FOLDERS):
            if any(
                (root / vers / folder / nasl_file.name) == nasl_file
                for vers in FEED_VERSIONS
            ):
                return

        yield LinterError(
            f"VT should be placed in the root directory ({root}).",
            file=nasl_file,
            plugin=self.name,
        )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re
from pathlib import Path
from typing import Iterator

from troubadix.helper import ScriptTag, get_script_tag_pattern
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckCVSSFormat(FileContentPlugin):
    name = "check_cvss_format"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        cvss_base_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)
        cvss_base_vector_pattern = get_script_tag_pattern(
            ScriptTag.CVSS_BASE_VECTOR
        )

        missing_cvss_base = re.search('"cvss_base", value:""', file_content)
        if missing_cvss_base:
            yield LinterError(
                "VT has a missing cvss_base value.",
                file=nasl_file,
                plugin=self.name,
            )
        else:
            cvss_detect = cvss_base_pattern.search(file_content)
            if not cvss_detect:
                yield LinterError(
                    "VT has an invalid cvss_base value.",
                    file=nasl_file,
                    plugin=self.name,
                )

        vector_match = cvss_base_vector_pattern.search(file_content)

        if not vector_match:
            yield LinterError(
                "VT has a missing or invalid cvss_base_vector value.",
                file=nasl_file,
                plugin=self.name,
            )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.helper import CURRENT_ENCODING
from troubadix.plugin import LinterError, LinterFix
from troubadix.plugins.copyright_text import (
    CORRECT_COPYRIGHT_PHRASE,
    CheckCopyrightText,
)

from . import PluginTestCase

WRONG_TEXTS = [
    "# Text descriptions are largely excerpted from the referenced\n"
    "# advisory, and are Copyright (C) the respective author(s)\n",
    "# Text descriptions are largely excerpted from the referenced\n"
    "# advisory, and are Copyright (C) the respective author(s)\n",
    "# Text descriptions are largely excerpted from the referenced\n"
    "# advisory, and are Copyright (C) the respective author(s)\n",
    "# Some text descriptions might be excerpted from the referenced\n"
    "# advisories, and are Copyright (C) by the respective right holder(s)\n",
]


class CheckCopyrightTextTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("tests/file.nasl")
        content = (
            "# Copyright (C) 2016 Greenbone AG\n"
            "# Some text descriptions might be excerpted from (a) referenced\n"
            "# source(s), and are Copyright (C) by the respective "
            "right holder(s).\n"
            'script_copyright("Copyright (C) 1234");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )

        plugin = CheckCopyrightText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckCopyrightText(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_missing_statement(self):
        path = Path("tests/file.nasl")

        content = (
            "# Text descriptions are largely excerpted from the referenced\n"
            "# advisory, and are Copyright (C) the respective author(s)\n"
            '  script_copyright("Copyright (C) 134");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )

        plugin = CheckCopyrightText(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "The VT is using an incorrect copyright statement.",
            results[0].message,
        )

    def test_wrong_copyright_text(self):
        path = Path("file.nasl")

        for wrong_text in WRONG_TEXTS:
            content = (
                "# Copyright (C) 2017 Greenbone AG\n"
                f"{wrong_text}"
                '  script_copyright("Copyright (C) 1234");'
            )
            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content
            )

            plugin = CheckCopyrightText(fake_context)

            results = list(plugin.run())
            self.assertEqual(len(results), 1)

            self.assertIsInstance(results[0], LinterError)
            self.assertEqual(
                "The VT is using an incorrect copyright statement.",
                results[0].message,
            )

    def test_fix_wrong_copyright_text(self):
        for wrong_text in WRONG_TEXTS:
            with self.create_directory() as tempdir:
                path = tempdir / "file.nasl"
                content = (
                    "# Copyright (C) 2017 Greenbone AG\n"
                    f"{wrong_text}"
                    '  script_copyright("Copyright (C) 1234");'
                )
                path.write_text(content, encoding=CURRENT_ENCODING)

                fake_context = self.create_file_plugin_context(
                    nasl_file=path, file_content=content
                )

                plugin = CheckCopyrightText(fake_context)

                list(plugin.run())

                results = list(plugin.fix())
                self.assertEqual(len(results), 1)

                self.assertIsInstance(results[0], LinterFix)
                self.assertEqual(
                    "The copyright statement has been updated to "
                    f"{CORRECT_COPYRIGHT_PHRASE}",
                    results[0].message,
                )



# Copyright (C) 2023 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import List

from troubadix.standalone_plugins.changed_packages.package import (
    Direction,
    Package,
    Reasons,
)

from .marker import Marker

PACKAGE_UPDATE_SUFFIX_PATTERN = re.compile(r"(?P<update>\d+)$")


class ChangedUpdate(Marker):
    @classmethod
    def mark(cls, missing_packages: List[Package], new_packages: List[Package]):
        for package in new_packages:
            match = PACKAGE_UPDATE_SUFFIX_PATTERN.search(package.version)
            if not match:
                continue

            suffix = match.group("update")
            other_package = next(
                (
                    old_package
                    for old_package in missing_packages
                    if package.name == old_package.name
                    and old_package.version.startswith(
                        package.version.replace(suffix, "")
                    )
                    and package.release == old_package.release
                ),
                None,
            )

            if not other_package:
                continue

            package.reasons[Reasons.CHANGED_UPDATE] = Direction.ACTIVE
            other_package.reasons[Reasons.CHANGED_UPDATE] = Direction.ACTIVE



# Copyright (C) 2021 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterable, Iterator

from ..helper import is_ignore_file
from ..plugin import LineContentPlugin, LinterResult, LinterWarning

_IGNORE_FILES = [
    "gb_openvas",
    "gb_gsa_",
    "http_func.inc",
    "misc_func.inc",
]


class CheckTodoTbd(LineContentPlugin):
    """This step checks if a given VT contains the words TODO, TBD
    or @todo as a comment.
    """

    name = "check_todo_tbd"

    def check_lines(
        self,
        nasl_file: Path,
        lines: Iterable[str],
    ) -> Iterator[LinterResult]:
        if is_ignore_file(nasl_file, _IGNORE_FILES):
            return

        for index, line in enumerate(lines, start=1):
            match = re.search("##? *(TODO|TBD|@todo):?", line)
            if match is not None:
                yield LinterWarning(
                    "VT contains #TODO/TBD/@todo keywords.",
                    file=nasl_file,
                    plugin=self.name,
                    line=index,
                )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from datetime import datetime
from pathlib import Path
from typing import Iterator

from troubadix.helper import ScriptTag, get_script_tag_pattern
from troubadix.plugin import (
    FileContentPlugin,
    LinterError,
    LinterResult,
    LinterWarning,
)


class CheckCVEFormat(FileContentPlugin):
    name = "check_cve_format"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        tag_pattern = get_script_tag_pattern(ScriptTag.CVSS_BASE)

        # don't need to check detection scripts since they don't refer to CVEs.
        # all detection scripts have a cvss of 0.0
        cvss_detect = tag_pattern.search(file_content)
        if cvss_detect and cvss_detect.group("value") == "0.0":
            return

        # ("CVE-2017-2750");
        match_result = re.search("(?<=script_cve_id)[^;]+", file_content)
        if match_result is None or match_result.group(0) is None:
            yield LinterWarning(
                "VT does not refer to any CVEs.",
                file=nasl_file,
                plugin=self.name,
            )
            return

        found_cves = []
        matches = match_result.group(0).split(",")
        current_year = datetime.now().year
        for match in matches:
            result = re.search(r'"CVE-\d{4}-\d{4,7}"', match)
            if result is None or result.group(0) is None:
                yield LinterError(
                    "VT uses an invalid CVE format.",
                    file=nasl_file,
                    plugin=self.name,
                )
                return

            cve = result.group(0)

            if len(cve) > 15 and cve[10] == "0":
                yield LinterError(
                    "The last group of CVE digits of the VT "
                    "must not start with a 0 if there are more than 4 digits.",
                    file=nasl_file,
                    plugin=self.name,
                )

            year = cve.split("-")
            if not 1999 <= int(year[1]) <= current_year:
                yield LinterError(
                    "VT uses an invalid year in CVE format.",
                    file=nasl_file,
                    plugin=self.name,
                )

            if cve in found_cves:
                yield LinterError(
                    f"VT is using CVE {cve} multiple times.",
                    file=nasl_file,
                    plugin=self.name,
                )

            found_cves.append(cve)



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import subprocess
import sys
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Iterable, List

from troubadix.helper.patterns import (
    LAST_MODIFICATION_ANY_VALUE_PATTERN,
    SCRIPT_VERSION_ANY_VALUE_PATTERN,
)
from troubadix.standalone_plugins.common import git

SCRIPT_VERSION_PATTERN = re.compile(
    r"^\+\s*" + SCRIPT_VERSION_ANY_VALUE_PATTERN, re.MULTILINE
)
SCRIPT_LAST_MODIFICATION_PATTERN = re.compile(
    r"^\+\s*" + LAST_MODIFICATION_ANY_VALUE_PATTERN, re.MULTILINE
)


def file_type(string: str) -> Path:
    file_path = Path(string)
    if not file_path.is_file():
        raise ValueError(f"{string} is not a file.")
    return file_path


def parse_args(args: Iterable[str]) -> Namespace:
    parser = ArgumentParser(
        description="Check for changed files that did not alter "
        "last_modification and script_version",
    )
    parser.add_argument(
        "-c",
        "--commit_range",
        type=str,
        required=True,
        help=(
            "Git commit range to check e.g "
            "2c87f4b6062804231fd508411510ca07fd270380^..HEAD or"
            "YOUR_BRANCH..main"
        ),
    )
    parser.add_argument(
        "-f",
        "--files",
        nargs="+",
        type=file_type,
        default=[],
        help=(
            "List of files to diff. "
            "If empty use all files added or modified in the commit range."
        ),
    )
    return parser.parse_args(args=args)


def check_version_updated(files: List[Path], commit_range: str) -> bool:
    """The script checks (via git diff) if the passed VT has changed the
    the following tags:

    - script_version("[...]");
    - script_tag(name:"last_modification", value:"[...]");
    """

    if not files:
        files = [
            Path(f)
            for f in git(
                "diff", "--name-only", "--diff-filter=d", commit_range
            ).splitlines()
        ]

    rcode = True
    for nasl_file in files:
        if nasl_file.suffix != ".nasl" or not nasl_file.exists():
            continue

        print(f"Check file {nasl_file}")
        text = git(
            "-c",
            "color.status=false",
            "--no-pager",
            "diff",
            commit_range,
            nasl_file,
        )

        if not SCRIPT_VERSION_PATTERN.search(text):
            print(
                f"{nasl_file}: Missing updated script_version", file=sys.stderr
            )
            rcode = False

        if not SCRIPT_LAST_MODIFICATION_PATTERN.search(text):
            print(
                f"{nasl_file}: Missing updated last_modification",
                file=sys.stderr,
            )
            rcode = False

    return rcode


def main() -> int:
    args = sys.argv[1:]

    try:
        git_base = git("rev-parse", "--show-toplevel")
        os.chdir(git_base.rstrip("\n"))
    except subprocess.SubprocessError:
        print(
            "Your current working directory doesn't belong to a git repository"
        )
        return 1

    parsed_args = parse_args(args)
    if not check_version_updated(parsed_args.files, parsed_args.commit_range):
        return 2

    return 0


if __name__ == "__main__":
    sys.exit(main())



import re
from argparse import ArgumentParser, Namespace
from subprocess import CalledProcessError
from typing import List, Set, Tuple

from pontos.terminal.terminal import ConsoleTerminal

from troubadix.argparser import file_type
from troubadix.helper.patterns import _get_special_script_tag_pattern
from troubadix.standalone_plugins.common import get_merge_base, git

CVE_PATTERN = re.compile(r"CVE-\d{4}-\d{4,}")


def compare(
    old_content: str, current_content: str
) -> Tuple[List[str], List[str]]:
    old_cves = get_cves_from_content(old_content)
    current_cves = get_cves_from_content(current_content)

    missing_cves = sorted(old_cves.difference(current_cves))
    added_cves = sorted(current_cves.difference(old_cves))

    return missing_cves, added_cves


def get_cves_from_content(content: str) -> Set[str]:
    pattern = _get_special_script_tag_pattern(
        name="cve_id", flags=re.MULTILINE | re.DOTALL
    )
    match = pattern.search(content)
    if not match:
        return set()

    cves = CVE_PATTERN.findall(match.group("value"))
    return set(cves)


def parse_args() -> Namespace:
    parser = ArgumentParser(
        description="Check for changed CVEs in VTs",
    )
    parser.add_argument(
        "--files",
        nargs="+",
        type=file_type,
        default=[],
        required=True,
        help="List of files to check.",
    )
    parser.add_argument(
        "--start-commit",
        type=str,
        required=False,
        help=(
            "The commit before the changes to check have been introduced. "
            "If the files have been renamed before, choose that commit. "
            "Defaults to the merge-base with main"
        ),
        default=get_merge_base("main", "HEAD"),
    )
    parser.add_argument(
        "--hide-equal",
        action="store_true",
        help="Omit log message, if a file has equal CVEs",
    )

    return parser.parse_args()


def main():
    args = parse_args()
    terminal = ConsoleTerminal()

    terminal.info(
        f"Checking {len(args.files)} file(s) from {args.start_commit} to HEAD"
    )

    for file in args.files:
        try:
            old_content = git("show", f"{args.start_commit}:{file}")
            current_content = git("show", f"HEAD:{file}")
        except CalledProcessError:
            terminal.error(
                f"Could not find {file} at {args.start_commit} or HEAD"
            )
            continue

        missing_cves, added_cves = compare(old_content, current_content)

        if not missing_cves and not added_cves:
            if not args.hide_equal:
                terminal.info(f"{file} has equal CVEs")
            continue

        terminal.warning(f"CVEs for {file} differ")
        if missing_cves:
            terminal.print("Missing CVEs: ", ", ".join(missing_cves))
        if added_cves:
            terminal.print("Added CVEs: ", ", ".join(added_cves))



#  Copyright (c) 2022 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.misplaced_compare_in_if import CheckMisplacedCompareInIf

from . import PluginTestCase


class CheckMisplacedCompareInIfTestCase(PluginTestCase):
    def test_ok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok2(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( "text" >< variable ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok3(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( "text" >< variable )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok4(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( "text" >!< variable ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok5(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( "text" >!< variable )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok6(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( "text" >< variable ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok7(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( "text" >< variable )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok8(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( "text" >!< variable ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok9(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( "text" >!< variable )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok10(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( "text" >< variable ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok11(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( "text" >< variable )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok12(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( "text" >!< variable ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ok13(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( "text" >!< variable )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_nok(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( variable >< "text" ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'if( variable >< "text" ) {',
            results[0].message,
        )

    def test_nok2(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( variable >< "text" )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'if( variable >< "text" )\nexit(1);',
            results[0].message,
        )

    def test_nok3(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( variable >!< "text" ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'if( variable >!< "text" ) {',
            results[0].message,
        )

    def test_nok4(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'if( variable >!< "text" )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'if( variable >!< "text" )\nexit(1);',
            results[0].message,
        )

    def test_nok5(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( variable >< "text" ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'else if( variable >< "text" ) {',
            results[0].message,
        )

    def test_nok6(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( variable >< "text" )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'else if( variable >< "text" )\nexit(1);',
            results[0].message,
        )

    def test_nok7(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( variable >!< "text" ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'else if( variable >!< "text" ) {',
            results[0].message,
        )

    def test_nok8(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            'else if( variable >!< "text" )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            'else if( variable >!< "text" )\nexit(1);',
            results[0].message,
        )

    def test_nok9(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( variable >< "text" ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            '} else if( variable >< "text" ) {',
            results[0].message,
        )

    def test_nok10(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( variable >< "text" )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            '} else if( variable >< "text" )\nexit(1);',
            results[0].message,
        )

    def test_nok11(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( variable >!< "text" ) {}\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            '} else if( variable >!< "text" ) {',
            results[0].message,
        )

    def test_nok12(self):
        nasl_file = Path(__file__).parent / "test.nasl"
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
            '} else if( variable >!< "text" )\nexit(1);\n'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=nasl_file, file_content=content
        )
        plugin = CheckMisplacedCompareInIf(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "VT/Include is using a misplaced compare "
            "within an if() call in "
            '} else if( variable >!< "text" )\nexit(1);',
            results[0].message,
        )



# Copyright (c) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.valid_oid import CheckValidOID

from . import PluginTestCase


class CheckValidOIDTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = 'script_oid("1.3.6.1.4.1.25623.1.0.100376");'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_empty_tag(self):
        path = Path("some/file.nasl")
        content = "script_oid();"
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            "No valid script_oid() call found",
            results[0].message,
        )

    def test_invalid_oid(self):
        path = Path("some/file.nasl")
        content = 'script_oid("1.3.6.1.4.1.25623.2.0.100376");'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an invalid "
                "OID '1.3.6.1.4.1.25623.2.0.100376'"
            ),
            results[0].message,
        )

    def test_missing__script_family(self):
        path = Path("some/file.nasl")
        content = 'script_oid("1.3.6.1.4.1.25623.1.1.100376");'
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual("VT is missing a script family!", results[0].message)

    def test_euler_family_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.2.2025.5555");'
            'script_family("Huawei EulerOS Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_euler_family(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.2.2055.5555");'
            'script_family("Huawei EulerOS Local Security Checks");'
        )

        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an invalid OID "
                "'1.3.6.1.4.1.25623.1.1.2.2055.5555' (EulerOS pattern:"
                " 1.3.6.1.4.1.25623.1.1.2.[ADVISORY_YEAR].[ADVISORY_ID])"
            ),
            results[0].message,
        )

    def test_suse_family_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.4.2025.55555.5");'
            'script_family("SuSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_suse_family(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.4.2025.555755.5");'
            'script_family("SuSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an invalid OID '1.3.6.1.4.1.25623.1.1."
                "4.2025.555755.5' (SLES pattern: 1.3.6.1.4.1.25623.1.1.4"
                ".[ADVISORY_YEAR].[ADVISORY_ID].[ADVISORY_REVISION])"
            ),
            results[0].message,
        )

    def test_debian_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.1.2256");'
            'script_family("Debian Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_debian(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.1.2256");'
            'script_family("Suse Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Debian '1.3.6.1.4.1.25623.1.1.1.2256'"
            ),
            results[0].message,
        )

    def test_unused_oid(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.3.2256");'
            'script_family("Suse Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an invalid OID '1.3.6.1.4.1.25623.1.1"
                ".3.2256' (Vendor OID with unknown Vendor-Prefix)"
            ),
            results[0].message,
        )

    def test_suse_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.4.2012.2256.1");'
            'script_family("SuSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_suse(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.4.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "SUSE SLES '1.3.6.1.4.1.25623.1.1.4.2256'"
            ),
            results[0].message,
        )

    def test_amazon_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.5.2012.2256");'
            'script_family("Amazon Linux Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_amazon(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.5.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Amazon Linux '1.3.6.1.4.1.25623.1.1.5.2256'"
            ),
            results[0].message,
        )

    def test_gentoo_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.6.2256");'
            'script_family("Gentoo Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_gentoo(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.6.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Gentoo '1.3.6.1.4.1.25623.1.1.6.2256'"
            ),
            results[0].message,
        )

    def test_freebsd_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.7.2256");'
            'script_family("FreeBSD Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_freebsd(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.7.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "FreeBSD '1.3.6.1.4.1.25623.1.1.7.2256'"
            ),
            results[0].message,
        )

    def test_oracle_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.8.2256");'
            'script_family("Oracle Linux Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_oracle(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.8.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Oracle Linux '1.3.6.1.4.1.25623.1.1.8.2256'"
            ),
            results[0].message,
        )

    def test_fedora_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.9.2256");'
            'script_family("Fedora Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_fedora(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.9.2256");'
            'script_family("Debian Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Fedora '1.3.6.1.4.1.25623.1.1.9.2256'"
            ),
            results[0].message,
        )

    def test_mageia_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.10.2012.2256");'
            'script_family("Mageia Linux Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_mageia(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.10.2256");'
            'script_family("Debian Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Mageia Linux '1.3.6.1.4.1.25623.1.1.10.2256'"
            ),
            results[0].message,
        )

    def test_redhat_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.11.2256");'
            'script_family("RedHat Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_redhat(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.11.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "RedHat '1.3.6.1.4.1.25623.1.1.11.2256'"
            ),
            results[0].message,
        )

    def test_ubuntu_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.12.2012.2256.1");'
            'script_family("Ubuntu Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_ubuntu(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.12.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Ubuntu '1.3.6.1.4.1.25623.1.1.12.2256'"
            ),
            results[0].message,
        )

    def test_slackware_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.13.2022.123.01");'
            'script_family("Slackware Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_slackware(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.13.2022.123.01");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Slackware '1.3.6.1.4.1.25623.1.1.13.2022.123.01'"
            ),
            results[0].message,
        )

    def test_rocky_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.14.2022.123");'
            'script_family("Rocky Linux Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_rocky(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.14.2022.123");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for "
                "Rocky Linux '1.3.6.1.4.1.25623.1.1.14.2022.123'"
            ),
            results[0].message,
        )

    def test_unknown(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.1.99.2256");'
            'script_family("SUSE Local Security Checks");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)

        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an invalid OID '1.3.6.1.4.1.25623.1.1."
                "99.2256' (Vendor OID with unknown Vendor-Prefix)"
            ),
            results[0].message,
        )

    def test_script_name__product_unknown(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.2.2025.2555");'
            'script_family("Huawei EulerOS Local Security Checks");'
            'script_name("AdaptBB Detection (HTTP)");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an invalid OID '1.3.6.1.4.1.25623.1.2"
                ".2025.2555' (last digits)"
            ),
            results[0].message,
        )

    def test_script_name__product_firefox_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.2.1.2020.255");'
            'script_name("Mozilla Firefox Security Advisory");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_script_name__product_firefox(self):
        path = Path("some/file.nasl")
        content = (
            'script_oid("1.3.6.1.4.1.25623.1.2.1.2020.255");'
            'script_name("AdaptBB Detection (HTTP)");'
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckValidOID(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 1)
        self.assertIsInstance(results[0], LinterError)
        self.assertEqual(
            (
                "script_oid() is using an OID that is reserved for 'Firefox' "
                "(1.3.6.1.4.1.25623.1.2.1.2020.255)"
            ),
            results[0].message,
        )



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.helper.helper import SCRIPT_CATEGORIES
from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckScriptCategory(FileContentPlugin):
    name = "check_script_category"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        if nasl_file.suffix == ".inc":
            return

        own_category_match = re.search(
            r"^\s*script_category\s*\(([^)]{3,})\)\s*;",
            file_content,
            re.MULTILINE,
        )

        if own_category_match is None or own_category_match.group(1) is None:
            yield LinterError(
                "VT is missing a script_category.",
                file=nasl_file,
                plugin=self.name,
            )
            return

        # pylint: disable=line-too-long
        # See https://github.com/greenbone/openvas-scanner/blob/master/misc/nvt_categories.h
        # for a list of the category numbers.
        own_category = own_category_match.group(1)
        if own_category not in SCRIPT_CATEGORIES:
            yield LinterError(
                f"VT is using an unsupported category '{own_category}'.",
                file=nasl_file,
                plugin=self.name,
            )
            return



# Copyright (C) 2022 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterError
from troubadix.plugins.deprecated_functions import CheckDeprecatedFunctions

from . import PluginTestCase


class CheckDeprecatedDependencyTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            "script_category(ACT_ATTACK);"
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, file_content=content
        )
        plugin = CheckDeprecatedFunctions(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_deprecated_functions(self):
        deprecated_output = {
            'script_summary(), use script_tag(name:"'
            'summary", value:"") instead': "script_"
            "summary('this is not okay!');\n",
            "script_id(), use script_oid() with "
            "the full OID instead": "script_id(123345);\n",
            "security_note()": "security_note('nonono!');\n",
            "security_warning()": "security_warning('nonono!');\n",
            "security_hole()": "security_hole('nonono!');",
            "script_description()": "script_description('stop it!')\n",
            'script_tag(name:"risk_factor", value: '
            "SEVERITY)": 'script_tag(name:"risk_factor", value: 0.1);\n',
            "script_bugtraq_id()": "script_bugtraq_id('00000');",
        }
        path = Path("some/file.nasl")
        for msg, cont in deprecated_output.items():
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                'script_tag(name:"summary", value:"Foo Bar.");\n'
                f"script_category(ACT_ATTACK);\n{cont}"
            )
            fake_context = self.create_file_plugin_context(
                nasl_file=path, file_content=content
            )
            plugin = CheckDeprecatedFunctions(fake_context)

            results = list(plugin.run())

            self.assertEqual(len(results), 1)
            self.assertIsInstance(results[0], LinterError)
            self.assertEqual(
                f"Found a deprecated function call: {msg}",
                results[0].message,
            )
